@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: grass57dataset.cpp 15584 2008-10-23 14:53:15Z warmerdam $
+ * $Id: grass57dataset.cpp 28291 2015-01-05 14:45:07Z martinl $
  *
  * Project:  GRASS Driver
  * Purpose:  Implement GRASS raster read/write support
@@ -10,6 +10,7 @@
  *
  ******************************************************************************
  * Copyright (c) 2000 Frank Warmerdam <warmerdam@pobox.com>
+ * Copyright (c) 2007-2010, Even Rouault <even dot rouault at mines-paris dot org>
  *
  * Permission is hereby granted, free of charge, to any person obtaining a
  * copy of this software and associated documentation files (the "Software"),
@@ -45,9 +46,15 @@ extern "C" {
 #include <grass/gprojects.h>
 #include <grass/gis.h>
 
-char *GPJ_grass_to_wkt(struct Key_Value *proj_info,
-		       struct Key_Value *proj_units,
-		       int esri_style, int prettify);
+#if GRASS_VERSION_MAJOR  >= 7
+char *GPJ_grass_to_wkt(const struct Key_Value *,
+		       const struct Key_Value *,
+		       int, int);
+#else
+char *GPJ_grass_to_wkt(struct Key_Value *,
+		       struct Key_Value *,
+		       int, int);
+#endif
 }
 
 #include "gdal_priv.h"
@@ -56,12 +63,38 @@ char *GPJ_grass_to_wkt(struct Key_Value *proj_info,
 
 #define GRASS_MAX_COLORS 100000  // what is the right value
 
-CPL_CVSID("$Id: grass57dataset.cpp 15584 2008-10-23 14:53:15Z warmerdam $");
+CPL_CVSID("$Id: grass57dataset.cpp 28291 2015-01-05 14:45:07Z martinl $");
 
 CPL_C_START
 void	GDALRegister_GRASS(void);
 CPL_C_END
 
+#if GRASS_VERSION_MAJOR  >= 7
+#define G_get_cellhd             Rast_get_cellhd
+#define G_raster_map_type        Rast_map_type
+#define G_read_fp_range          Rast_read_fp_range
+#define G_get_fp_range_min_max   Rast_get_fp_range_min_max
+#define G_set_c_null_value       Rast_set_c_null_value
+#define G_set_f_null_value       Rast_set_f_null_value
+#define G_set_d_null_value       Rast_set_d_null_value
+#define G_open_cell_old          Rast_open_old
+#define G_copy                   memcpy
+#define G_read_colors            Rast_read_colors
+#define G_get_color_range        Rast_get_c_color_range
+#define G_colors_count           Rast_colors_count
+#define G_get_f_color_rule       Rast_get_fp_color_rule
+#define G_free_colors            Rast_free_colors
+#define G_close_cell             Rast_close
+#define G_allocate_c_raster_buf  Rast_allocate_c_buf
+#define G_get_c_raster_row       Rast_get_c_row
+#define G_is_c_null_value        Rast_is_c_null_value
+#define G_get_f_raster_row       Rast_get_f_row
+#define G_get_d_raster_row       Rast_get_d_row
+#define G_allocate_f_raster_buf  Rast_allocate_f_buf
+#define G_allocate_d_raster_buf  Rast_allocate_d_buf
+#define G__setenv                G_setenv_nogisrc
+#endif
+
 /************************************************************************/
 /*                         Grass2CPLErrorHook()                         */
 /************************************************************************/
@@ -308,7 +341,11 @@ GRASSRasterBand::GRASSRasterBand( GRASSDataset *poDS, int nBand,
             int	nRed, nGreen, nBlue;
             GDALColorEntry    sColor;
 
+#if GRASS_VERSION_MAJOR  >= 7
+            if( Rast_get_c_color( &iColor, &nRed, &nGreen, &nBlue, &sGrassColors ) )
+#else
             if( G_get_color( iColor, &nRed, &nGreen, &nBlue, &sGrassColors ) )
+#endif
             {
                 sColor.c1 = nRed;
                 sColor.c2 = nGreen;
@@ -713,7 +750,7 @@ GRASSDataset::~GRASSDataset()
     if ( pszElement )
 	G_free ( pszElement );
 
-    CPLFree( pszProjection );
+    G_free( pszProjection );
 }
 
 /************************************************************************/
@@ -790,7 +827,7 @@ bool GRASSDataset::SplitPath( char *path, char **gisdbase, char **location,
 /*                                Open()                                */
 /************************************************************************/
 
-#if GRASS_VERSION_MAJOR  >= 6 && GRASS_VERSION_MINOR  >= 3
+#if (GRASS_VERSION_MAJOR  >= 6 && GRASS_VERSION_MINOR  >= 3) || GRASS_VERSION_MAJOR  >= 7
 typedef int (*GrassErrorHandler)(const char *, int);
 #else
 typedef int (*GrassErrorHandler)(char *, int);
@@ -803,8 +840,6 @@ GDALDataset *GRASSDataset::Open( GDALOpenInfo * poOpenInfo )
     char	*pszMapset = NULL, *pszElem = NULL, *pszName = NULL;
     char        **papszCells = NULL;
     char        **papszMapsets = NULL;
-    static char fake_gisbase[50];
-    static bool hasGisbase;
 
 /* -------------------------------------------------------------------- */
 /*      Does this even look like a grass file path?                     */
@@ -825,15 +860,20 @@ GDALDataset *GRASSDataset::Open( GDALOpenInfo * poOpenInfo )
     // Set error function
     G_set_error_routine ( (GrassErrorHandler) Grass2CPLErrorHook );
     
-    
+
+    // GISBASE is path to the directory where GRASS is installed,
     if ( !getenv( "GISBASE" ) ) {
-	// we are outside a GRASS session
-	// TODO: use function instead of hardcoded path
-	sprintf(fake_gisbase, "GISBASE=/usr/local/share/gdal/grass/" );	
-        putenv( fake_gisbase );
-	hasGisbase = false;
-    } else {
-	hasGisbase = true;
+        static char* gisbaseEnv = NULL;
+        const char *gisbase = GRASS_GISBASE;
+        CPLError( CE_Warning, CPLE_AppDefined, "GRASS warning: GISBASE "
+                "enviroment variable was not set, using:\n%s", gisbase );
+        char buf[2000];
+        snprintf ( buf, sizeof(buf), "GISBASE=%s", gisbase );
+        buf[sizeof(buf)-1] = '\0';
+
+        CPLFree(gisbaseEnv);
+        gisbaseEnv = CPLStrdup ( buf );
+        putenv( gisbaseEnv );
     }
 
     if ( !SplitPath( poOpenInfo->pszFilename, &pszGisdb, &pszLoc, &pszMapset,
@@ -918,12 +958,16 @@ GDALDataset *GRASSDataset::Open( GDALOpenInfo * poOpenInfo )
 /* -------------------------------------------------------------------- */
 /*      Capture some information from the file that is of interest.     */
 /* -------------------------------------------------------------------- */
-    
+
+#if GRASS_VERSION_MAJOR  >= 7
+    Rast_get_cellhd( papszCells[0], papszMapsets[0], &(poDS->sCellInfo) );
+#else
     if( G_get_cellhd( papszCells[0], papszMapsets[0], &(poDS->sCellInfo) ) != 0 ) {
-	CPLError( CE_Warning, CPLE_AppDefined, "GRASS: Cannot open raster header");
-	delete poDS;
-	return NULL;
+        CPLError( CE_Warning, CPLE_AppDefined, "GRASS: Cannot open raster header");
+        delete poDS;
+        return NULL;
     }
+#endif
 
     poDS->nRasterXSize = poDS->sCellInfo.cols;
     poDS->nRasterYSize = poDS->sCellInfo.rows;
@@ -940,13 +984,11 @@ GDALDataset *GRASSDataset::Open( GDALOpenInfo * poOpenInfo )
 /* -------------------------------------------------------------------- */
     struct Key_Value *projinfo, *projunits;
 
-    if ( hasGisbase ) {
-	projinfo = G_get_projinfo();
-	projunits = G_get_projunits();
-        poDS->pszProjection = GPJ_grass_to_wkt ( projinfo, projunits, 0, 0);
-        G_free_key_value(projinfo);
-        G_free_key_value(projunits);
-    }
+    projinfo = G_get_projinfo();
+    projunits = G_get_projunits();
+    poDS->pszProjection = GPJ_grass_to_wkt ( projinfo, projunits, 0, 0);
+    if (projinfo) G_free_key_value(projinfo);
+    if (projunits) G_free_key_value(projunits);
 
 /* -------------------------------------------------------------------- */
 /*      Create band information objects.                                */
@@ -968,7 +1010,19 @@ GDALDataset *GRASSDataset::Open( GDALOpenInfo * poOpenInfo )
 
     CSLDestroy(papszCells);
     CSLDestroy(papszMapsets);
-
+    
+/* -------------------------------------------------------------------- */
+/*      Confirm the requested access is supported.                      */
+/* -------------------------------------------------------------------- */
+    if( poOpenInfo->eAccess == GA_Update )
+    {
+        delete poDS;
+        CPLError( CE_Failure, CPLE_NotSupported, 
+                  "The GRASS driver does not support update access to existing"
+                  " datasets.\n" );
+        return NULL;
+    }
+    
     return poDS;
 }
 
