@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: ogrgrasslayer.cpp,v 1.3 2006/02/14 13:01:49 rblazek Exp $
+ * $Id: ogrgrasslayer.cpp 27044 2014-03-16 23:41:27Z rouault $
  *
  * Project:  OpenGIS Simple Features Reference Implementation
  * Purpose:  Implements OGRGRASSLayer class.
@@ -7,6 +7,7 @@
  *
  ******************************************************************************
  * Copyright (c) 2005, Radim Blazek <radim.blazek@gmail.com>
+ * Copyright (c) 2008-2010, Even Rouault <even dot rouault at mines-paris dot org>
  *
  * Permission is hereby granted, free of charge, to any person obtaining a
  * copy of this software and associated documentation files (the "Software"),
@@ -25,26 +26,13 @@
  * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
  * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
  * DEALINGS IN THE SOFTWARE.
- ******************************************************************************
- *
- * $Log: ogrgrasslayer.cpp,v $
- * Revision 1.3  2006/02/14 13:01:49  rblazek
- * dont kill on win
- *
- * Revision 1.2  2005/09/21 00:59:36  fwarmerdam
- * fixup OGRFeatureDefn and OGRSpatialReference refcount handling
- *
- * Revision 1.1  2005/08/05 15:32:43  fwarmerdam
- * New
- *
- *
- */
+ ****************************************************************************/
 
 #include <signal.h>
 #include "ogrgrass.h"
 #include "cpl_conv.h"
 
-CPL_CVSID("$Id: ogrgrasslayer.cpp,v 1.3 2006/02/14 13:01:49 rblazek Exp $");
+CPL_CVSID("$Id: ogrgrasslayer.cpp 27044 2014-03-16 23:41:27Z rouault $");
 
 /************************************************************************/
 /*                           OGRGRASSLayer()                            */
@@ -111,20 +99,27 @@ OGRGRASSLayer::OGRGRASSLayer( int layerIndex,  struct Map_info * map )
 	types |= type;
         CPLDebug ( "GRASS", "type = %d types = %d", type, types );
     }
+    
+    OGRwkbGeometryType eGeomType = wkbUnknown;
     if ( types == GV_LINE || types == GV_BOUNDARY || types == GV_LINES ) 
     {
-        poFeatureDefn->SetGeomType ( wkbLineString );
+        eGeomType = wkbLineString;
     } 
     else if ( types == GV_POINT )
     {
-	poFeatureDefn->SetGeomType ( wkbPoint );
+        eGeomType = wkbPoint;
     }
     else if ( types == GV_AREA )
     {
         CPLDebug ( "GRASS", "set wkbPolygon" );
-	poFeatureDefn->SetGeomType ( wkbPolygon );
+        eGeomType = wkbPolygon;
     }
 
+    if (Vect_is_3d(poMap))
+        poFeatureDefn->SetGeomType ( (OGRwkbGeometryType)(eGeomType | wkb25DBit) );
+    else
+        poFeatureDefn->SetGeomType ( eGeomType );
+
     // Get attributes definition
     poDbString = (dbString*) CPLMalloc ( sizeof(dbString) );
     poCursor = (dbCursor*) CPLMalloc ( sizeof(dbCursor) );
@@ -160,7 +155,7 @@ OGRGRASSLayer::OGRGRASSLayer( int layerIndex,  struct Map_info * map )
 			    ogrFtype = OFTString;
 			    break; 
 			 case DB_C_TYPE_DATETIME:
-			    ogrFtype = OFTString;
+			    ogrFtype = OFTDateTime;
 			    break; 
 		    }
 
@@ -218,8 +213,11 @@ OGRGRASSLayer::OGRGRASSLayer( int layerIndex,  struct Map_info * map )
 	if ( srsWkt ) 
 	{
 	    poSRS = new OGRSpatialReference ( srsWkt );
-	    CPLFree ( srsWkt );
+	    G_free ( srsWkt );
 	}
+
+        G_free_key_value(projinfo);
+        G_free_key_value(projunits);
     }
 }
 
@@ -248,7 +246,7 @@ OGRGRASSLayer::~OGRGRASSLayer()
     
     if ( paFeatureIndex ) CPLFree ( paFeatureIndex );
     
-    if ( poLink ) CPLFree ( poLink );
+    if ( poLink ) G_free ( poLink );
     
     Vect_destroy_line_struct ( poPoints );
     Vect_destroy_cats_struct ( poCats );
@@ -643,7 +641,11 @@ bool OGRGRASSLayer::SetSpatialMatch()
 	
 	Vect_cidx_get_cat_by_index ( poMap, iLayerIndex, cidx, &cat, &type, &id );
 
+#if GRASS_VERSION_MAJOR  >= 7
+    struct bound_box box;
+#else
 	BOUND_BOX box;
+#endif
 
 	switch ( type ) 
 	{
@@ -784,7 +786,7 @@ OGRFeature *OGRGRASSLayer::GetNextFeature()
 OGRFeature *OGRGRASSLayer::GetFeature( long nFeatureId )
 
 {
-    CPLDebug ( "GRASS", "OGRGRASSLayer::GetFeature nFeatureId = %d", nFeatureId );
+    CPLDebug ( "GRASS", "OGRGRASSLayer::GetFeature nFeatureId = %ld", nFeatureId );
 
     int cat;
     OGRFeature *poFeature = NULL;
@@ -860,7 +862,7 @@ OGRFeature *OGRGRASSLayer::GetFeature( long nFeatureId )
 /************************************************************************/
 OGRGeometry *OGRGRASSLayer::GetFeatureGeometry ( long nFeatureId, int *cat )
 {
-    CPLDebug ( "GRASS", "OGRGRASSLayer::GetFeatureGeometry nFeatureId = %d", nFeatureId );
+    CPLDebug ( "GRASS", "OGRGRASSLayer::GetFeatureGeometry nFeatureId = %ld", nFeatureId );
 
     int cidx = paFeatureIndex[(int)nFeatureId];
 
@@ -870,12 +872,16 @@ OGRGeometry *OGRGRASSLayer::GetFeatureGeometry ( long nFeatureId, int *cat )
     //CPLDebug ( "GRASS", "cat = %d type = %d id = %d", *cat, type, id );
 
     OGRGeometry *poOGR = NULL;
+    int bIs3D = Vect_is_3d(poMap);
 
     switch ( type ) {
 	case GV_POINT:
         {
 	    Vect_read_line ( poMap, poPoints, poCats, id);
-	    poOGR = new OGRPoint( poPoints->x[0], poPoints->y[0], poPoints->z[0] );
+            if (bIs3D)
+                poOGR = new OGRPoint( poPoints->x[0], poPoints->y[0], poPoints->z[0] );
+            else
+                poOGR = new OGRPoint( poPoints->x[0], poPoints->y[0] );
         }
         break;
 	    
@@ -884,8 +890,12 @@ OGRGeometry *OGRGRASSLayer::GetFeatureGeometry ( long nFeatureId, int *cat )
         {
 	    Vect_read_line ( poMap, poPoints, poCats, id);
 	    OGRLineString *poOGRLine = new OGRLineString();
-            poOGRLine->setPoints( poPoints->n_points, 
-		                  poPoints->x, poPoints->y, poPoints->z );
+            if (bIs3D)
+                poOGRLine->setPoints( poPoints->n_points, 
+                                      poPoints->x, poPoints->y, poPoints->z );
+            else
+                poOGRLine->setPoints( poPoints->n_points, 
+                                      poPoints->x, poPoints->y );
 
             poOGR = poOGRLine;
         }
@@ -900,8 +910,12 @@ OGRGeometry *OGRGRASSLayer::GetFeatureGeometry ( long nFeatureId, int *cat )
 
 	    OGRLinearRing       *poRing;
 	    poRing = new OGRLinearRing();
-	    poRing->setPoints( poPoints->n_points,
-		               poPoints->x, poPoints->y, poPoints->z ); 
+            if (bIs3D)
+                poRing->setPoints( poPoints->n_points,
+                                poPoints->x, poPoints->y, poPoints->z );
+            else
+                poRing->setPoints( poPoints->n_points,
+                                poPoints->x, poPoints->y ); 
 
 	    poOGRPoly->addRingDirectly( poRing );
 
@@ -912,8 +926,12 @@ OGRGeometry *OGRGRASSLayer::GetFeatureGeometry ( long nFeatureId, int *cat )
 		Vect_get_isle_points ( poMap, isle, poPoints );
 
 		poRing = new OGRLinearRing();
-		poRing->setPoints( poPoints->n_points,
-				   poPoints->x, poPoints->y, poPoints->z ); 
+                if (bIs3D)
+                    poRing->setPoints( poPoints->n_points,
+                                    poPoints->x, poPoints->y, poPoints->z );
+                else
+                    poRing->setPoints( poPoints->n_points,
+                                    poPoints->x, poPoints->y );
 
 		poOGRPoly->addRingDirectly( poRing );
 	    }
@@ -1014,7 +1032,11 @@ int OGRGRASSLayer::GetFeatureCount( int bForce )
 /************************************************************************/
 OGRErr OGRGRASSLayer::GetExtent (OGREnvelope *psExtent, int bForce)
 {
+#if GRASS_VERSION_MAJOR  >= 7
+    struct bound_box box;
+#else
     BOUND_BOX box;
+#endif
 
     Vect_get_map_box ( poMap, &box );
 
